@@ -1,4 +1,4 @@
-/*  $Id: weather-icon.c 2782 2007-05-26 20:04:05Z nick $
+/*  $Id: weather-icon.c 7550 2009-06-13 07:46:50Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  *
@@ -44,9 +44,13 @@ get_icon (const gchar *number,
 
   image = gdk_pixbuf_new_from_file_at_scale (filename, size, size, TRUE, NULL);
 
-  if (G_UNLIKELY (!image))
+  if (G_UNLIKELY (!image)) {
     g_warning ("Unable to open image: %s", filename);
-
+    if (number && strcmp(number, "99")) {
+      g_free(filename);
+      return get_icon("99", size);
+    }
+  }
   g_free (filename);
 
   return image;
