@@ -1,4 +1,4 @@
-/*  $Id: weather-scrollbox.c 2555 2007-03-14 20:07:53Z nick $
+/*  $Id: weather-scrollbox.c 7748 2009-07-19 20:57:42Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  *
@@ -437,7 +437,7 @@ gtk_scrollbox_expose (GtkWidget      *widget,
     gdk_draw_drawable (widget->window,
                        widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
                        self->pixmap,
-                       0, self->draw_offset,
+                       0, (self->animate ? self->draw_offset : 0),
                        self->draw_middle, 0,
                        widget->allocation.width, widget->allocation.height);
 
@@ -445,6 +445,12 @@ gtk_scrollbox_expose (GtkWidget      *widget,
 }
 
 
+void
+gtk_scrollbox_next_label(GtkScrollbox *self)
+{
+  stop_callback (self);
+  start_draw_down (self);
+}
 
 void
 gtk_scrollbox_enablecb (GtkScrollbox *self,
@@ -546,3 +552,8 @@ gtk_scrollbox_clear (GtkScrollbox *self)
   self->draw_middle = 0;
   self->draw_maxmiddle = 0;
 }
+
+void gtk_scrollbox_set_animate(GtkScrollbox *self, gboolean animate)
+{ 
+  self->animate = animate;
+}
