@@ -1,4 +1,4 @@
-/*  $Id: weather-search.h 3212 2007-09-12 20:08:14Z nick $
+/*  $Id: weather-search.h 7609 2009-06-18 17:29:09Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  *
@@ -31,12 +31,17 @@ typedef struct
   GtkWidget    *dialog;
   GtkWidget    *search_entry;
   GtkWidget    *result_list;
+  GtkWidget    *find_button;
   GtkListStore *result_mdl;
+  GtkTreeViewColumn *column;
 
   gchar        *result;
+  gchar        *result_name;
 
   gchar        *proxy_host;
   gint          proxy_port;
+  
+  gchar        *last_search;
 }
 search_dialog;
 
@@ -44,6 +49,10 @@ search_dialog *create_search_dialog (GtkWindow *, gchar *, gint);
 
 gboolean run_search_dialog (search_dialog * dialog);
 
+gboolean weather_search_by_ip(const gchar *proxy_host, gint proxy_port,
+	void (*gui_cb)(const gchar *loc_name, const gchar *loc_code, gpointer user_data),
+	gpointer user_data);
+
 void free_search_dialog (search_dialog * dialog);
 
 G_END_DECLS
