@@ -1,4 +1,4 @@
-/*  $Id: weather-translate.c 2431 2007-01-19 14:23:08Z nick $
+/*  $Id: weather-translate.c 7687 2009-07-07 04:56:44Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  * 
@@ -32,7 +32,7 @@
 
 
 #define HDATE_N    (sizeof(gchar) * 100)
-#define DAY_LOC_N  (sizeof(gchar) * 20)
+#define DAY_LOC_N  (sizeof(gchar) * 100)
 #define TIME_LOC_N (sizeof(gchar) * 20)
 
 
@@ -85,6 +85,7 @@ static const gchar *desc_strings[] = {
   N_("Haze"),
   N_("Hazy"),
   N_("Heavy Rain"),
+  N_("Heavy Rain Shower"),
   N_("Heavy Snow"),
   N_("Hot And Humid"),
   N_("Hot!"),
@@ -122,6 +123,7 @@ static const gchar *desc_strings[] = {
   N_("PM Light Rain"),
   N_("PM Light Snow"),
   N_("PM Rain / Wind"),
+  N_("PM Rain / Snow Showers"),
   N_("PM Showers"),
   N_("PM Snow Showers"),
   N_("PM T-Storms"),
@@ -130,6 +132,7 @@ static const gchar *desc_strings[] = {
   N_("Rain / Snow / Wind"),
   N_("Rain / Snow Late"),
   N_("Rain / Snow Showers"),
+  N_("Rain / Snow Showers Early"),
   N_("Rain / Thunder"),
   N_("Rain / Wind"),
   N_("Rain and Sleet"),
@@ -154,6 +157,7 @@ static const gchar *desc_strings[] = {
   N_("Snow and Rain"),
   N_("Snow or Rain"),
   N_("Snow Shower"),
+  N_("Light Snow Shower"),
   N_("Snow Shower / Wind"),
   N_("Snow Showers Early"),
   N_("Snow Showers early"),
@@ -163,9 +167,11 @@ static const gchar *desc_strings[] = {
   N_("Sunny / Wind"),
   N_("T-Showers"),
   N_("T-Storm"),
+  N_("T-Storms"),
   N_("T-Storms / Wind"),
   N_("T-Storms Early"),
   N_("Thunder"),
+  N_("Thunder in the Vicinity"),
   N_("Thunder in the Vincinity"),
   N_("Variable Cloudiness"),
   N_("Variable Clouds"),
@@ -214,6 +220,7 @@ static const gchar *wdirs[] = {
   N_("ESE"),
   N_("SE"),
   N_("SSE"),
+  N_("CALM"),
   NULL
 };
 
@@ -302,8 +309,9 @@ translate_lsup (const gchar *lsup)
   gchar      *hdate;
   struct tm   time;
   gint        size = 0, i = 0;
- gchar      **lsup_split;
-
+  gchar      **lsup_split;
+  int          len;
+  
   if (lsup == NULL || strlen (lsup) == 0)
     return NULL;
 
@@ -332,8 +340,13 @@ translate_lsup (const gchar *lsup)
     {
       hdate = g_malloc (HDATE_N);
 
-      strftime (hdate, HDATE_N, _("%x at %X Local Time"), &time);
-
+      len = strftime (hdate, HDATE_N, _("%x at %X Local Time"), &time);
+      hdate[len] = 0;
+      if (!g_utf8_validate(hdate, -1, NULL)) {
+         gchar *utf8 = g_locale_to_utf8(hdate, -1, NULL, NULL, NULL);
+	 g_free(hdate);
+	 hdate = utf8;
+      }
       return hdate;
     }
   else
@@ -350,6 +363,7 @@ translate_day (const gchar *day)
   guint        i;
   const gchar *days[] = {"su", "mo", "tu", "we", "th", "fr", "sa", NULL};
   gchar       *day_loc;
+  int          len;
 
   if (day == NULL || strlen (day) < 2)
     return NULL;
@@ -366,7 +380,13 @@ translate_day (const gchar *day)
 
       day_loc = g_malloc (DAY_LOC_N);
 
-      strftime (day_loc, DAY_LOC_N, "%A", &time);
+      len = strftime (day_loc, DAY_LOC_N, "%A", &time);
+      day_loc[len] = 0;
+      if (!g_utf8_validate(day_loc, -1, NULL)) {
+         gchar *utf8 = g_locale_to_utf8(day_loc, -1, NULL, NULL, NULL);
+	 g_free(day_loc);
+	 day_loc = utf8;
+      }
 
       return day_loc;
     }
@@ -444,6 +464,7 @@ translate_time (const gchar *time)
   gchar     **time_split, *time_loc;
   gint        i = 0, size = 0;
   struct tm   time_tm;
+  int          len;
 
   if (strlen (time) == 0)
     return NULL;
@@ -461,7 +482,13 @@ translate_time (const gchar *time)
   _fill_time (&time_tm, time_split[0], time_split[1], time_split[2]);
   g_strfreev (time_split);
 
-  strftime (time_loc, TIME_LOC_N, "%X", &time_tm);
+  len = strftime (time_loc, TIME_LOC_N, "%X", &time_tm);
+  time_loc[len] = 0;
+  if (!g_utf8_validate(time_loc, -1, NULL)) {
+     gchar *utf8 = g_locale_to_utf8(time_loc, -1, NULL, NULL, NULL);
+     g_free(time_loc);
+     time_loc = utf8;
+  }
 
   return time_loc;
 }
