@@ -1,4 +1,4 @@
-/*  $Id: weather.c 7546 2009-06-13 07:35:18Z colin $
+/*  $Id: weather.c 7767 2009-07-23 09:11:00Z colin $
  *
  *  Copyright (c) 2003-2007 Xfce Development Team
  *
@@ -42,23 +42,33 @@
 #define UPDATE_TIME      1600
 #define BORDER           8
 
+#if !GLIB_CHECK_VERSION(2,14,0)
+#define g_timeout_add_seconds(t,c,d) g_timeout_add((t)*1000,(c),(d))
+#endif
+
 gboolean
 check_envproxy (gchar **proxy_host,
                 gint   *proxy_port)
 {
 
-  gchar *env_proxy = getenv ("HTTP_PROXY"), *tmp, **split;
+  gchar *env_proxy = NULL, *tmp, **split;
+
+  env_proxy = getenv ("HTTP_PROXY");
+  if (!env_proxy)
+    env_proxy = getenv ("http_proxy");
 
   if (!env_proxy)
     return FALSE;
 
   tmp = strstr (env_proxy, "://");
 
-  if (!tmp || strlen (tmp) < 3)
+  if (!tmp)
+    tmp = env_proxy;
+  else if (strlen (tmp) >= 3)
+    env_proxy = tmp + 3;
+  else
     return FALSE;
 
-  env_proxy = tmp + 3;
-
   /* we don't support username:password so return */
   tmp = strchr (env_proxy, '@');
   if (tmp)
@@ -162,6 +172,9 @@ make_label (xml_weather    *weatherdata,
     case WIND_GUST:
       value = translate_wind_speed (rawvalue, unit);
       break;
+    case BAR_D:
+      value = g_strdup(translate_bard(rawvalue));
+      break;
     default:
       value = NULL;
       break;
@@ -307,7 +320,8 @@ set_icon_current (xfceweather_data *data)
 
 static void
 cb_update (gboolean  succeed,
-           gchar    *result,
+           void    *result,
+	   size_t    len,
            gpointer  user_data)
 {
   xfceweather_data *data = user_data;
@@ -317,7 +331,12 @@ cb_update (gboolean  succeed,
 
   if (G_LIKELY (succeed && result))
     {
-      doc = xmlParseMemory (result, strlen (result));
+      if (g_utf8_validate(result, -1, NULL)) {
+	/* force parsing as UTF-8, the XML encoding header may lie */
+	doc = xmlReadMemory (result, strlen (result), NULL, "UTF-8", 0);
+      } else {
+	doc = xmlParseMemory (result, strlen(result));
+      }
 
       g_free (result);
 
@@ -439,6 +458,16 @@ xfceweather_read_config (XfcePanelPlugin  *plugin,
       data->location_code = g_strdup (value);
     }
 
+  value = xfce_rc_read_entry (rc, "loc_name", NULL);
+
+  if (value)
+    {
+      if (data->location_name)
+        g_free (data->location_name);
+
+      data->location_name = g_strdup (value);
+    }
+
   if (xfce_rc_read_bool_entry (rc, "celcius", TRUE))
     data->unit = METRIC;
   else
@@ -477,6 +506,11 @@ xfceweather_read_config (XfcePanelPlugin  *plugin,
       data->proxy_port = data->saved_proxy_port;
     }
 
+  data->animation_transitions = xfce_rc_read_bool_entry (rc, 
+  		"animation_transitions", TRUE);
+
+  gtk_scrollbox_set_animate(GTK_SCROLLBOX(data->scrollbox), data->animation_transitions);
+
   data->labels = labels_clear (data->labels);
 
   for (i = 0; i < 100 /* arbitrary */ ; ++i)
@@ -522,6 +556,9 @@ xfceweather_write_config (XfcePanelPlugin  *plugin,
   if (data->location_code)
     xfce_rc_write_entry (rc, "loc_code", data->location_code);
 
+  if (data->location_name)
+    xfce_rc_write_entry (rc, "loc_name", data->location_name);
+
   xfce_rc_write_bool_entry (rc, "proxy_fromenv", data->proxy_fromenv);
 
   if (data->proxy_host)
@@ -531,6 +568,8 @@ xfceweather_write_config (XfcePanelPlugin  *plugin,
       xfce_rc_write_int_entry (rc, "proxy_port", data->proxy_port);
     }
 
+  xfce_rc_write_bool_entry (rc, "animation_transitions", data->animation_transitions);
+
   for (i = 0; i < data->labels->len; i++)
     {
       g_snprintf (label, 10, "label%d", i);
@@ -553,7 +592,7 @@ update_weatherdata_with_reset (xfceweather_data *data)
   update_weatherdata (data);
 
   data->updatetimeout =
-    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_weatherdata, data);
+    g_timeout_add_seconds (UPDATE_TIME, (GSourceFunc) update_weatherdata, data);
 }
 
 
@@ -576,6 +615,27 @@ close_summary (GtkWidget *widget,
 
 
 
+static void
+forecast_click (GtkWidget *widget,
+          gpointer   user_data)
+{
+  xfceweather_data *data = (xfceweather_data *) user_data;
+
+  if (data->summary_window != NULL)
+    {
+
+      gtk_window_present (GTK_WINDOW (data->summary_window));
+    }
+  else
+    {
+      data->summary_window = create_summary_window (data);
+      g_signal_connect (G_OBJECT (data->summary_window), "destroy",
+                	G_CALLBACK (close_summary), data);
+
+      gtk_widget_show_all (data->summary_window);
+    }
+}
+
 static gboolean
 cb_click (GtkWidget      *widget,
           GdkEventButton *event,
@@ -585,21 +645,7 @@ cb_click (GtkWidget      *widget,
 
   if (event->button == 1)
     {
-
-      if (data->summary_window != NULL)
-        {
-
-          gtk_window_present (GTK_WINDOW (data->summary_window));
-        }
-      else
-        {
-          data->summary_window = create_summary_window (data->weatherdata,
-                                                        data->unit);
-          g_signal_connect (G_OBJECT (data->summary_window), "destroy",
-                            G_CALLBACK (close_summary), data);
-
-          gtk_widget_show_all (data->summary_window);
-        }
+      forecast_click(widget, user_data);
     }
   else if (event->button == 2)
     {
@@ -609,6 +655,20 @@ cb_click (GtkWidget      *widget,
   return FALSE;
 }
 
+static gboolean
+cb_scroll (GtkWidget      *widget,
+           GdkEventScroll *event,
+           gpointer        user_data)
+{
+  xfceweather_data *data = (xfceweather_data *) user_data;
+
+  if (event->direction == GDK_SCROLL_UP || event->direction == GDK_SCROLL_DOWN)
+    {
+      gtk_scrollbox_next_label(GTK_SCROLLBOX(data->scrollbox));
+    }
+
+  return FALSE;
+}
 
 
 static void
@@ -726,7 +786,7 @@ static xfceweather_data *
 xfceweather_create_control (XfcePanelPlugin *plugin)
 {
   xfceweather_data *data = panel_slice_new0 (xfceweather_data);
-  GtkWidget        *refresh;
+  GtkWidget        *refresh, *forecast;
   datas             lbl;
   GdkPixbuf        *icon = NULL;
 
@@ -772,6 +832,8 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
 
   g_signal_connect (G_OBJECT (data->tooltipbox), "button-press-event",
                     G_CALLBACK (cb_click), data);
+  g_signal_connect (G_OBJECT (data->tooltipbox), "scroll-event",
+                    G_CALLBACK (cb_scroll), data);
   gtk_widget_add_events(data->scrollbox, GDK_BUTTON_PRESS_MASK);
 
   /* add refresh button to right click menu, for people who missed the middle mouse click feature */
@@ -783,6 +845,20 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
 
   xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (refresh));
 
+  /* add refresh button to right click menu, for people who missed the middle mouse click feature */
+  forecast = gtk_image_menu_item_new_with_mnemonic (_("_Forecast"));
+  icon = get_icon ("32", 16);
+  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(forecast),
+  	gtk_image_new_from_pixbuf(icon));
+  if (G_LIKELY (icon))
+    g_object_unref (G_OBJECT (icon));
+  gtk_widget_show (forecast);
+
+  g_signal_connect (G_OBJECT (forecast), "activate",
+                    G_CALLBACK (forecast_click), data);
+
+  xfce_panel_plugin_menu_insert_item (plugin, GTK_MENU_ITEM (forecast));
+
   /* assign to tempval because g_array_append_val () is using & operator */
   lbl = TEMP;
   g_array_append_val (data->labels, lbl);
@@ -797,7 +873,7 @@ xfceweather_create_control (XfcePanelPlugin *plugin)
   gtk_scrollbox_clear (GTK_SCROLLBOX (data->scrollbox));
 
   data->updatetimeout =
-    gtk_timeout_add (UPDATE_TIME * 1000, (GSourceFunc) update_weatherdata, data);
+    g_timeout_add_seconds (UPDATE_TIME, (GSourceFunc) update_weatherdata, data);
 
   return data;
 }
@@ -830,6 +906,7 @@ xfceweather_free (XfcePanelPlugin  *plugin,
 
   /* Free chars */
   g_free (data->location_code);
+  g_free (data->location_name);
   g_free (data->proxy_host);
 
   /* Free Array */
